from django import forms
from django.contrib import admin

from shares.models import Share
from shares.views import hash_password


class ShareForm(forms.ModelForm):
    class Meta:
        model = Share
        exclude = ['pwd_hash']

    password = forms.CharField(label='New password', required=False)

    def save(self, *args, **kwargs):
        v = super().save(*args, **kwargs)
        if self.cleaned_data.get('password'):
            self.instance.pwd_hash = hash_password(self.cleaned_data['password'])
            self.instance.save()
        return v


@admin.register(Share)
class ShareAdmin(admin.ModelAdmin):
    form = ShareForm
    readonly_fields = 'share_type',
    list_select_related = 'creator',

    def has_add_permission(self, request):
        """Block user from adding new shares - you can use the /accounts/panel link to do that :D"""
        return False

    def delete_queryset(self, request, queryset):
        """
        Since Django does not call each instance's delete on mass delete in admin panel
        we need to do that ourselves.
        """
        for share in queryset:
            share.delete()
